# Copyright 2016-2018 PeppyMeter peppy.player@gmail.com
# 
# This file is part of PeppyMeter.
# 
# PeppyMeter is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# 
# PeppyMeter is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# 
# You should have received a copy of the GNU General Public License
# along with PeppyMeter. If not, see <http://www.gnu.org/licenses/>.

import time
import math
import threading

from threading import Thread

class CircularAnimator(Thread):
    """ Provides needle circular animation in a separate thread """
    
    def __init__(self, data_source, component, base, ui_refresh_period, needle_rects, get_data_method):
        """ Initializer
        
        :param data_source: data source
        :param channel: number of channels
        :param component: UI component
        :param base: meter base
        :param ui_refresh_period: animation interval 
        :param needle_rects: list of sprite rectangles
        :param get_data_method: method to get data
        """
        Thread.__init__(self)             
        self.data_source = data_source
        self.component = component
        self.run_flag = True
        self.base = base
        self.ui_refresh_period = ui_refresh_period
        self.previous_index = 0
        self.needle_rects = needle_rects
        self.get_data = get_data_method
        
    def run(self):
        """ Thread method. Converts volume value into the needle angle and displays corresponding sprite. """
        
        #print("run circ thread")
        while self.run_flag:
            #print("loop circ thread")
            volume = self.get_data()
            #print(threading.currentThread().native_id,"circ",volume,self.ui_refresh_period)

            #time.sleep(self.ui_refresh_period)
            #continue 

            if volume == None:
                volume = 0.0            
            #print("arrow1 ",volume,self.base.max_volume,self.base.incr)       
            n = (volume * self.base.max_volume * self.base.incr) / 100.0
            if n >= len(self.base.needle_sprites): n = len(self.base.needle_sprites) - 1
            
            if self.previous_index == int(n):
                #print("equal",self.previous_index,"->",n)       
                time.sleep(self.ui_refresh_period)
                continue
            
            diff = n - self.previous_index
            #sub_steps = range(int(abs(diff)) * self.base.steps_per_degree)
            #print("arrow1",self.previous_index,"->",n,"steps",sub_steps)       
            #sub_steps = range(0,int(abs(diff)) * self.base.steps_per_degree,int(abs(diff) * self.base.steps_per_degree/2)+1)
            #print("arrow2",self.previous_index,"->",n,"steps",sub_steps)       

            sign = int(math.copysign(1, diff))
            #for m in sub_steps:
            ##for m in [int(abs(diff)) * self.base.steps_per_degree]:
            #    #print("arrow act",m)       
            #    if not self.run_flag:
            #        break
            #    
            m=int(abs(diff)) * self.base.steps_per_degree;


            previous_rect = self.component.bounding_box
            self.base.draw_bgr_fgr(previous_rect, self.base.bgr)  
            next_index = (self.previous_index * self.base.steps_per_degree) + (m * sign)
            if next_index >= len(self.base.needle_sprites): next_index = len(self.base.needle_sprites) - 1
            
            sprite = self.base.needle_sprites[next_index]
            self.component.content = ("", sprite)
            r = self.needle_rects[next_index]
            self.component.content_x = r.x
            self.component.content_y = r.y
            r.x = 0
            r.y = 0
            self.component.bounding_box = r
            self.component.draw()
            r.x = self.component.content_x
            r.y = self.component.content_y

            a = previous_rect.union(r)
            if self.base.fgr:
                self.base.draw_bgr_fgr(a, self.base.fgr)

            self.base.update_rectangle(a)
            time.sleep(self.ui_refresh_period)

            self.previous_index = int(n)

